@@ -13,6 +13,7 @@
 #define REGEXPSTATEBUILDER_H
 
 #include <regexpstate.h>
+#include <list>
 
 class RegExpStatePointer {
   public:
@@ -29,6 +30,17 @@ build a RegExpState starting from language definitions
 */
 class RegExpStateBuilder
 {
+  private:
+    /*
+      FIXME
+      This is a trick to make shared_ptr to nested states and formatters
+      still reachable.
+    */
+    typedef std::list<RegExpFormatterPtr> RegExpFormatterList;
+    RegExpFormatterList nested_formatters;
+    typedef std::list<RegExpStatePtr> RegExpStateList;
+    RegExpStateList nested_states;
+    
   protected:
     // the following will be processed by doublecpp
     branches build
